#ifndef _SIMD_H_INCLUDED
#define _SIMD_H_INCLUDED
#include<stdio.h>

/******************************************************************************/
/*                        Declare the SIMD operations                         */
/******************************************************************************/

typedef int intv8 __attribute__ ((__mode__(__V8SI__)));
typedef unsigned uintv8 __attribute__ ((__mode__(__V8SI__)));
typedef float floatv4 __attribute__ ((__mode__(__V4SF__)));
typedef double doublev4 __attribute__ ((__mode__(__V4DF__)));
typedef int int256 __attribute__ ((__mode__(__V1OI__)));
typedef unsigned uint256 __attribute__ ((__mode__(__V1OI__)));
typedef long longv4 __attribute__ ((__mode__(__V4DI__)));
typedef unsigned long ulongv4 __attribute__ ((__mode__(__V4DI__)));
#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
typedef char charv32 __attribute__ ((__mode__(__V32QI__)));
typedef unsigned char ucharv32 __attribute__ ((__mode__(__V32QI__)));
typedef short shortv16 __attribute__ ((__mode__(__V16HI__)));
typedef unsigned short ushortv16 __attribute__ ((__mode__(__V16HI__)));
#endif
#endif

/******************************************************************************/
/*                        Load and Sotre Instruction                          */
/******************************************************************************/

#define simd_load(dest,src)                   \
({                                   	      \
  (dest) = __builtin_sw_load ((src));         \
  (dest);                                     \
})
#define simd_load_u(dest,src)			\
({						\
	(dest) = __builtin_sw_load_u ((src));	\
	(dest);                                 \
})
#define simd_loadu(dest,src)                  \
({                                            \
  (dest) = __builtin_sw_loadu ((src));        \
  (dest);                                     \
})
#define simd_loade(dest,src)                  \
({                                            \
   (dest) = __builtin_sw_loade ((src));       \
   (dest);                                    \
})
#define simd_vload_nc(dest,src)               \
({                                            \
  (dest) = __builtin_sw_vload_nc ((src));     \
  (dest);                                     \
})

#define simd_store(src,dest) __builtin_sw_store (dest, src)
#define simd_storeu(src,dest) __builtin_sw_storeu (dest, src)
#define simd_store_u(src,dest) __builtin_sw_store_u (dest, src)
#define simd_storeul(src,dest) __builtin_sw_storeul (dest, src)
#define simd_storeuh(src,dest) __builtin_sw_storeuh (dest, src)
#define simd_vstore_nc(src,dest) __builtin_sw_vstore_nc (dest, src)

/******************************************************************************/
/*                        INT Arithmetic Instruction                          */
/******************************************************************************/

//vaddw vsubw vaddl subl
static __inline intv8 __attribute__ ((__always_inline__)) 
simd_vaddw (intv8 __A, intv8 __B) 
{
  return (intv8) (__A + __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vsubw (intv8 __A, intv8 __B)
{
  return (intv8) (__A - __B);
}

#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
static __inline longv4 __attribute__((__always_inline__))
simd_vaddl (longv4 __A, longv4 __B)
{
	return (longv4)  (__A + __B);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vsubl (longv4 __A, longv4 __B)
{
	return (longv4)  (__A - __B);
}
#else
static __inline int256 __attribute__ ((__always_inline__))
simd_vaddl (int256 __A, int256 __B)
{
  return (int256) (__A + __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_vsubl (int256 __A, int256 __B)
{
  return (int256) (__A - __B);
}
#endif
#endif

//ctpopow
static __inline int __attribute__ ((__always_inline__))
simd_ctpopow (int256 __A)
{
  return (int) __builtin_sw_ctpopow (__A);
}
//ctlzow
static __inline int __attribute__ ((__always_inline__))
simd_ctlzow (int256 __A) 
{
  return (int) __builtin_sw_ctlzow (__A);
}
//vucaddw
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucaddw (intv8 __A, intv8 __B)
{
  return (intv8) __builtin_sw_vucaddw (__A, __B);
}
//vucaddwi
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucaddwi (intv8 __A, const int __B)
{
  return (intv8) __builtin_sw_vucaddwi (__A, __B);
}
//vucsubw
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucsubw (intv8 __A, intv8 __B)
{
  return (intv8) __builtin_sw_vucsubw (__A, __B);
}
//vucsubwi
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucsubwi (intv8 __A, const int __B)
{
  return (intv8) __builtin_sw_vucsubwi (__A, __B);
}

#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
//vucaddh vucaddhi vucsubh vucsubhi  vucaddb  vucaddbi vucsubb vucsubbi
static __inline shortv16 __attribute__((__always_inline__))
simd_vucaddh (shortv16 __A, shortv16 __B)
{
	return (shortv16)  __builtin_sw_vucaddh_v16hi(__A, __B);
}
static __inline shortv16 __attribute__((__always_inline__))
simd_vucaddhi (shortv16 __A, const int __B)
{
	return (shortv16)  __builtin_sw_vucaddhi_v16hi(__A, __B);
}

static __inline shortv16 __attribute__((__always_inline__))
simd_vucsubh (shortv16 __A, shortv16 __B)
{
	return (shortv16)  __builtin_sw_vucsubh_v16hi(__A, __B);
}
static __inline shortv16 __attribute__((__always_inline__))
simd_vucsubhi (shortv16 __A, const int __B)
{
	return (shortv16)  __builtin_sw_vucsubhi_v16hi(__A, __B);
}

static __inline charv32 __attribute__((__always_inline__))
simd_vucaddb (charv32 __A, charv32 __B)
{
	return (charv32)  __builtin_sw_vucaddb_v32qi(__A, __B);
}
static __inline charv32 __attribute__((__always_inline__))
simd_vucaddbi (charv32 __A, const int __B)
{
	return (charv32)  __builtin_sw_vucaddbi_v32qi(__A, __B);
}

static __inline charv32 __attribute__((__always_inline__))
simd_vucsubb (charv32 __A, charv32 __B)
{
	return (charv32)  __builtin_sw_vucsubb_v32qi(__A, __B);
}
static __inline charv32 __attribute__((__always_inline__))
simd_vucsubbi (charv32 __A, const int __B)
{
	return (charv32)  __builtin_sw_vucsubbi_v32qi(__A, __B);
}

static __inline int __attribute__((__always_inline__))
simd_vsumw (intv8 __A)
{
        return (int) __builtin_sw_vsumw(__A);
}

static __inline long __attribute__((__always_inline__))
simd_vsuml (longv4 __A)
{
        return (long) __builtin_sw_vsuml(__A);
}
#else
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucaddh (intv8 __A, intv8 __B)
{
  return (intv8) __builtin_sw_vucaddh (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucaddhi (intv8 __A, const int __B)
{
  return (intv8) __builtin_sw_vucaddhi (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucsubh (intv8 __A, intv8 __B)
{
  return (intv8) __builtin_sw_vucsubh (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucsubhi (intv8 __A, const int __B)
{
  return (intv8) __builtin_sw_vucsubhi (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucaddb (intv8 __A, intv8 __B)
{
  return (intv8) __builtin_sw_vucaddb (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucaddbi (intv8 __A, const int __B)
{
  return (intv8) __builtin_sw_vucaddbi (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucsubb (intv8 __A, intv8 __B)
{
  return (intv8) __builtin_sw_vucsubb (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vucsubbi (intv8 __A, const int __B)
{
  return (intv8) __builtin_sw_vucsubbi (__A, __B);
}
#endif 
#endif

//vsllw
//vsllwi
#ifdef XUKK20230920_SWSIMD
static __inline intv8 __attribute__ ((__always_inline__))
simd_vsllw (intv8 __A, int __B)
{
  return (intv8) __builtin_sw_vsllw (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vsllwi (intv8 __A, const int __B)
{
  return (intv8) __builtin_sw_vsllwi (__A, __B);
}

static __inline intv8 __attribute__ ((__always_inline__))
simd_vsrlw (intv8 __A, int __B)
{
  return (intv8) __builtin_sw_vsrlw (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vsrlwi (intv8 __A, const int __B)
{
  return (intv8) __builtin_sw_vsrlwi (__A, __B);
}

static __inline intv8 __attribute__ ((__always_inline__))
simd_vsraw (intv8 __A, int __B)
{
  return (intv8) __builtin_sw_vsraw (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vsrawi (intv8 __A, const int __B)
{
  return (intv8) __builtin_sw_vsrawi (__A, __B);
}
#endif

static __inline uintv8 __attribute__((__always_inline__))
simd_vsllw (uintv8 __A, int __B) 
{
        return (uintv8)  (__A << __B);
}
//#define simd_vsllwi(x,y) x<<y
static __inline uintv8 __attribute__((__always_inline__))
simd_vsllwi (uintv8 __A, const int __B) 
{
        return (uintv8)  (__A << __B);
}
//vsrlw
//vsrlwi
static __inline uintv8 __attribute__((__always_inline__))
simd_vsrlw (uintv8 __A, int __B) 
{
        return (uintv8) (__A >> __B);
}
//#define simd_vsrlwi(x,y) x>>y
static __inline uintv8 __attribute__((__always_inline__))
simd_vsrlwi (uintv8 __A, const int __B) 
{
        return (uintv8)  (__A >> __B);
}
//vsraw
//vsrawi
static __inline intv8 __attribute__((__always_inline__))
simd_vsraw (intv8 __A, int __B) 
{
        return (intv8)  (__A >> __B);
}
static __inline intv8 __attribute__((__always_inline__))
simd_vsrawi (intv8 __A, const int __B) 
{
        return (intv8)  (__A >> __B);
}
//vrolw
//vrolwi
static __inline intv8 __attribute__ ((__always_inline__))
simd_vrolw (intv8 __A, int __B)
{
  return (intv8) __builtin_sw_vrolw (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vrolwi (intv8 __A, const int __B)
{
  return (intv8) __builtin_sw_vrolwi (__A, __B);
}
//sllow
//sllowi
static __inline int256 __attribute__ ((__always_inline__)) 
simd_sllow (int256 __A, int __B)
{
  return (int256) __builtin_sw_sllow (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_sllowi (int256 __A, const int __B)
{
  return (int256) __builtin_sw_sllowi (__A, __B);
}

#ifndef XUKK20231013_SWSIMD
//srlow srlowi
static __inline int256 __attribute__ ((__always_inline__))
simd_srlow (int256 __A, int __B)
{
  return (int256) __builtin_sw_srlow (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_srlowi (int256 __A, const int __B)
{
  return (int256) __builtin_sw_srlowi (__A, __B);
}
#else
static __inline int256 __attribute__ ((__always_inline__))
simd_sllow (int256 __A, int __B) 
{
        return (int256)  (__A<<__B);
}
static __inline int256 __attribute__((__always_inline__))
simd_sllowi (int256 __A, const int __B) 
{
        return (int256)  (__A<<__B);
}

static __inline int256 __attribute__ ((__always_inline__))
simd_srlow (int256 __A, int __B) 
{
        return (int256)  (__A>>__B);
}
static __inline int256 __attribute__((__always_inline__))
simd_srlowi (int256 __A, const int __B) 
{
        return (int256)  (__A>>__B);
}
#endif

#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
//vsllb vsllbi
static __inline charv32 __attribute__((__always_inline__))
simd_vsllb (charv32 __A, int __B)
{
	return (charv32)(__A << __B);
}
static __inline charv32 __attribute__((__always_inline__))
simd_vsllbi (charv32 __A, const int __B)
{
	return (charv32)(__A << __B);
}

//vsrlb vsrlbi
static __inline ucharv32 __attribute__((__always_inline__))
simd_vsrlb (ucharv32 __A, int __B)
{
	return (ucharv32)(__A >> __B);
}
static __inline ucharv32 __attribute__((__always_inline__))
simd_vsrlbi (ucharv32 __A, const int __B)
{
	return (ucharv32)(__A >> __B);
}

//vsrab vsrabi
static __inline charv32 __attribute__((__always_inline__))
simd_vsrab (charv32 __A, int __B)
{
	return (charv32)(__A >> __B);
}
static __inline charv32 __attribute__((__always_inline__))
simd_vsrabi (charv32 __A, const int __B)
{
	return (charv32)(__A >> __B);
}

//vrolb vrolbi
static __inline charv32 __attribute__((__always_inline__))
simd_vrolb (charv32 __A, int __B)
{
	return (charv32)__builtin_sw_vrolb(__A, __B);
}
static __inline charv32 __attribute__((__always_inline__))
simd_vrolbi (charv32 __A, const int __B)
{
	return (charv32)__builtin_sw_vrolbi(__A, __B);
}

//vsllh vsllhi
static __inline shortv16 __attribute__((__always_inline__))
simd_vsllh (shortv16 __A, int __B)
{
	return (shortv16)(__A << __B);
}
static __inline shortv16 __attribute__((__always_inline__))
simd_vsllhi (shortv16 __A, const int __B)
{
	return (shortv16)(__A << __B);
}

//vsrlh  vsrlhi
static __inline ushortv16 __attribute__((__always_inline__))
simd_vsrlh (ushortv16 __A, int __B)
{
	return (ushortv16)(__A >> __B);
}
static __inline ushortv16 __attribute__((__always_inline__))
simd_vsrlhi (ushortv16 __A, const int __B)
{
	return (ushortv16)(__A >> __B);
}

//vsrah  vsrahi
static __inline shortv16 __attribute__((__always_inline__))
simd_vsrah (shortv16 __A, int __B)
{
	return (shortv16)(__A >> __B);
}
static __inline shortv16 __attribute__((__always_inline__))
simd_vsrahi (shortv16 __A, const int __B)
{
	return (shortv16)(__A >> __B);
}

//vrolh vrolhi
static __inline shortv16 __attribute__((__always_inline__))
simd_vrolh (shortv16 __A, int __B)
{
	return (shortv16)__builtin_sw_vrolh(__A, __B);
}
static __inline shortv16 __attribute__((__always_inline__))
simd_vrolhi (shortv16 __A, const int __B)
{
	return (shortv16)__builtin_sw_vrolhi(__A, __B);
}

//vslll vsllli
static __inline longv4 __attribute__((__always_inline__))
simd_vslll (longv4 __A, int __B)
{
	return (longv4) (__A << __B);
}
static __inline longv4 __attribute__((__always_inline__))
simd_vsllli (longv4 __A, const int __B)
{
	return (longv4) (__A << __B);
}

//vsrll vsrlli
static __inline ulongv4 __attribute__((__always_inline__))
simd_vsrll (ulongv4 __A, int __B)
{
	return (ulongv4) (__A >> __B);
}
static __inline ulongv4 __attribute__((__always_inline__))
simd_vsrlli (ulongv4 __A, const int __B)
{
	return (ulongv4) (__A >> __B);
}

//vsral vsrali
static __inline longv4 __attribute__((__always_inline__))
simd_vsral (longv4 __A, int __B)
{
	return (longv4) (__A >> __B);
}
static __inline longv4 __attribute__((__always_inline__))
simd_vsrali (longv4 __A, const int __B)
{
	return (longv4) (__A >> __B);
}

//vroll vrolli
static __inline longv4 __attribute__((__always_inline__))
simd_vroll (longv4 __A, int __B)
{
	return (longv4)__builtin_sw_vroll(__A, __B);
}
static __inline longv4 __attribute__((__always_inline__))
simd_vrolli (longv4 __A, const int __B)
{
	return (longv4)__builtin_sw_vrolli(__A, __B);
}

//sraow sraowi
static __inline int256 __attribute__((__always_inline__))
simd_sraow (int256 __A, int __B) 
{
        return (int256) __builtin_sw_sraow(__A, __B);
}
static __inline int256 __attribute__((__always_inline__))
simd_sraowi (int256 __A, const int __B) 
{
        return (int256)  __builtin_sw_sraowi(__A, __B);
}
#endif
#endif

//vcmpgew
static __inline int __attribute__ ((__always_inline__))
simd_vcmpgew (intv8 __A, intv8 __B)
{
  return (int) __builtin_sw_vcmpgew (__A, __B);
}
//vcmpgewi
static __inline int __attribute__ ((__always_inline__))
simd_vcmpgewi (intv8 __A, const int __B)
{
  return (int) __builtin_sw_vcmpgewi (__A, __B);
}
//vcmpeqw
static __inline intv8 __attribute__((__always_inline__))
simd_vcmpeqw (intv8 __A, intv8 __B)
{
        return (intv8)(__A == __B);
}
//vcmpeqwi
static __inline intv8 __attribute__((__always_inline__))
simd_vcmpeqwi (intv8 __A, const int __B)
{
        return (intv8)(__A == __B);
}
//vcmplew
static __inline intv8 __attribute__((__always_inline__))
simd_vcmplew (intv8 __A, intv8 __B)
{
        return (intv8)(__A <= __B);
}
//vcmplewi
static __inline intv8 __attribute__((__always_inline__))
simd_vcmplewi (intv8 __A, const int __B)
{
        return (intv8)(__A <= __B);
}
//vcmpltw
static __inline intv8 __attribute__((__always_inline__))
simd_vcmpltw (intv8 __A, intv8 __B)
{
        return (intv8)(__A < __B);
}
//vcmpltwi
static __inline intv8 __attribute__((__always_inline__))
simd_vcmpltwi (intv8 __A, const int __B)
{
        return (intv8)(__A < __B);
}
//vcmpulew
static __inline uintv8 __attribute__((__always_inline__))
simd_vcmpulew (uintv8 __A, uintv8 __B)
{
        return (uintv8)(__A <= __B);
}
//vcmpulewi
static __inline uintv8 __attribute__((__always_inline__))
simd_vcmpulewi (uintv8 __A, const unsigned int __B)
{
        return (uintv8)(__A <= __B);
}
//vcmpultw
static __inline uintv8 __attribute__((__always_inline__))
simd_vcmpultw (uintv8 __A, uintv8 __B)
{
        return (uintv8)(__A < __B);
}
//vcmpultwi
static __inline uintv8 __attribute__((__always_inline__))
simd_vcmpultwi (uintv8 __A, const unsigned int __B)
{
        return (uintv8)(__A < __B);
}

#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
//vcmpueqb vcmpeuqbi
static __inline ucharv32 __attribute__((__always_inline__))
simd_vcmpueqb (ucharv32 __A, ucharv32 __B)
{
        return (ucharv32)__builtin_sw_vcmpueqb(__A, __B);
}
static __inline ucharv32 __attribute__((__always_inline__))
simd_vcmpueqbi (ucharv32 __A, const unsigned int __B)
{
        return (ucharv32)__builtin_sw_vcmpueqbi(__A, __B);
}

//vcmpugtb vcmpugtbi
static __inline ucharv32 __attribute__((__always_inline__))
simd_vcmpugtb (ucharv32 __A, ucharv32 __B)
{
        return (ucharv32)__builtin_sw_vcmpugtb(__A, __B);
}
static __inline ucharv32 __attribute__((__always_inline__))
simd_vcmpugtbi (ucharv32 __A, const unsigned int __B)
{
        return (ucharv32)__builtin_sw_vcmpugtbi(__A, __B);
}

//vmaxb
static __inline charv32 __attribute__((__always_inline__))
simd_vmaxb (charv32 __A, charv32 __B)
{
	return (charv32)  __builtin_sw_vmaxb(__A, __B);
}

//vminb
static __inline charv32 __attribute__((__always_inline__))
simd_vminb (charv32 __A, charv32 __B)
{
	return (charv32)  __builtin_sw_vminb(__A, __B);
}

//vmaxh
static __inline shortv16 __attribute__((__always_inline__))
simd_vmaxh (shortv16 __A, shortv16 __B)
{
	return (shortv16)  __builtin_sw_vmaxh(__A, __B);
}

//vminh
static __inline shortv16 __attribute__((__always_inline__))
simd_vminh (shortv16 __A, shortv16 __B)
{
	return (shortv16)  __builtin_sw_vminh(__A, __B);
}

//vmaxl
static __inline longv4 __attribute__((__always_inline__))
simd_vmaxl (longv4 __A, longv4 __B)
{
	return (longv4)  __builtin_sw_vmaxl(__A, __B);
}

//vminl
static __inline longv4 __attribute__((__always_inline__))
simd_vminl (longv4 __A, longv4 __B)
{
	return (longv4)  __builtin_sw_vminl(__A, __B);
}

//vumaxb
static __inline ucharv32 __attribute__((__always_inline__))
simd_vumaxb (ucharv32 __A, ucharv32 __B)
{
	return (ucharv32)  __builtin_sw_vumaxb(__A, __B);
}

//vuminb
static __inline ucharv32 __attribute__((__always_inline__))
simd_vuminb (ucharv32 __A, ucharv32 __B)
{
	return (ucharv32)  __builtin_sw_vuminb(__A, __B);
}

//vumaxh
static __inline ushortv16 __attribute__((__always_inline__))
simd_vumaxh (ushortv16 __A, ushortv16 __B)
{
	return (ushortv16)  __builtin_sw_vumaxh(__A, __B);
}

//vuminh
static __inline ushortv16 __attribute__((__always_inline__))
simd_vuminh (ushortv16 __A, ushortv16 __B)
{
	return (ushortv16)  __builtin_sw_vuminh(__A, __B);
}

//vumaxl
static __inline ulongv4 __attribute__((__always_inline__))
simd_vumaxl (ulongv4 __A, ulongv4 __B)
{
	return (ulongv4)  __builtin_sw_vumaxl(__A, __B);
}

//vuminl
static __inline ulongv4 __attribute__((__always_inline__))
simd_vuminl (ulongv4 __A, ulongv4 __B)
{
	return (ulongv4)  __builtin_sw_vuminl(__A, __B);
}
#endif
//vmaxw
static __inline intv8 __attribute__((__always_inline__))
simd_vmaxw (intv8 __A, intv8 __B)
{
	return (intv8)  __builtin_sw_vmaxw(__A, __B);
}

//vminw
static __inline intv8 __attribute__((__always_inline__))
simd_vminw (intv8 __A, intv8 __B)
{
	return (intv8)  __builtin_sw_vminw(__A, __B);
}

//vumaxw
static __inline uintv8 __attribute__((__always_inline__))
simd_vumaxw (uintv8 __A, uintv8 __B)
{
	return (uintv8)  __builtin_sw_vumaxw(__A, __B);
}

//vuminw
static __inline uintv8 __attribute__((__always_inline__))
simd_vuminw (uintv8 __A, uintv8 __B)
{
	return (uintv8)  __builtin_sw_vuminw(__A, __B);
}
#endif

//
//vseleqw
static __inline intv8 __attribute__((__always_inline__))
simd_vseleqw (intv8 __A, intv8 __B, intv8 __C)
{
        return (intv8)__builtin_sw_vseleqw(__A, __B, __C);
}
//vseleqwi
static __inline intv8 __attribute__((__always_inline__))
simd_vseleqwi (intv8 __A, intv8 __B, const int __C)
{
        return (intv8)__builtin_sw_vseleqwi(__A, __B, __C);
}
//vsellbcw
static __inline intv8 __attribute__((__always_inline__))
simd_vsellbcw (intv8 __A, intv8 __B, intv8 __C)
{
        return (intv8)__builtin_sw_vsellbcw(__A, __B, __C);
}
//vsellbcwi
static __inline intv8 __attribute__((__always_inline__))
simd_vsellbcwi (intv8 __A, intv8 __B, const int __C)
{
        return (intv8)__builtin_sw_vsellbcwi(__A, __B, __C);
}
//vselltw
static __inline intv8 __attribute__((__always_inline__))
simd_vselltw (intv8 __A, intv8 __B, intv8 __C)
{
        return (intv8)__builtin_sw_vselltw(__A, __B, __C);
}
//vselltwi
static __inline intv8 __attribute__((__always_inline__))
simd_vselltwi (intv8 __A, intv8 __B, const int __C)
{
        return (intv8)__builtin_sw_vselltwi(__A, __B, __C);
}
//vsellew
static __inline intv8 __attribute__((__always_inline__))
simd_vsellew (intv8 __A, intv8 __B, intv8 __C)
{
        return (intv8)__builtin_sw_vsellew(__A, __B, __C);
}
//vsellewi
static __inline intv8 __attribute__((__always_inline__))
simd_vsellewi (intv8 __A, intv8 __B, const int __C)
{
        return (intv8)__builtin_sw_vsellewi(__A, __B, __C);
}
//
//simd_vlog
static __inline intv8 __attribute__ ((__always_inline__))
simd_vlog (intv8 __x, intv8 __y, intv8 __z, const int i)
{
  return (intv8) __builtin_inst_opt_vlogzz_v8si (__x, __y, __z, i);
}
#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
//vandb vandh vandl
static __inline charv32 __attribute__((__always_inline__))
simd_vandb (charv32 __A, charv32 __B)
{
        return (charv32)  (__A & __B);
}

static __inline shortv16 __attribute__((__always_inline__))
simd_vandh (shortv16 __A, shortv16 __B)
{
        return (shortv16)  (__A & __B);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vandl (longv4 __A, longv4 __B)
{
        return (longv4)  (__A & __B);
}

//vbicb vbich vbicl
static __inline charv32 __attribute__((__always_inline__))
simd_vbicb (charv32 __A, charv32 __B)
{
        return (charv32)  (__A &~ __B);
}

static __inline shortv16 __attribute__((__always_inline__))
simd_vbich (shortv16 __A, shortv16 __B)
{
        return (shortv16)  (__A &~ __B);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vbicl (longv4 __A, longv4 __B)
{
        return (longv4)  (__A &~ __B);
}

//vbisb vbish vbisl
static __inline charv32 __attribute__((__always_inline__))
simd_vbisb (charv32 __A, charv32 __B)
{
        return (charv32)  (__A | __B);
}

static __inline shortv16 __attribute__((__always_inline__))
simd_vbish (shortv16 __A, shortv16 __B)
{
        return (shortv16)  (__A | __B);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vbisl (longv4 __A, longv4 __B)
{
        return (longv4)  (__A | __B);
}

//vornotb vornoth vornotl
static __inline charv32 __attribute__((__always_inline__))
simd_vornotb (charv32 __A, charv32 __B)
{
        return (charv32)  (__A |~ __B);
}

static __inline shortv16 __attribute__((__always_inline__))
simd_vornoth (shortv16 __A, shortv16 __B)
{
        return (shortv16)  (__A |~ __B);
}
static __inline longv4 __attribute__((__always_inline__))
simd_vornotl (longv4 __A, longv4 __B)
{
        return (longv4)  (__A |~ __B);
}

//vxorb vxorh vxorl
static __inline charv32 __attribute__((__always_inline__))
simd_vxorb (charv32 __A, charv32 __B)
{
        return (charv32)  (__A ^ __B);
}

static __inline shortv16 __attribute__((__always_inline__))
simd_vxorh (shortv16 __A, shortv16 __B)
{
        return (shortv16)  (__A ^ __B);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vxorl (longv4 __A, longv4 __B)
{
        return (longv4)  (__A ^ __B);
}
//veqvb veqvh veqvl
static __inline charv32 __attribute__((__always_inline__))
simd_veqvb (charv32 __A, charv32 __B)
{
        return (charv32)__builtin_sw_veqvb(__A, __B);
}

static __inline shortv16 __attribute__((__always_inline__))
simd_veqvh (shortv16 __A, shortv16 __B)
{
        return (shortv16)__builtin_sw_veqvh(__A, __B);
}

static __inline longv4 __attribute__((__always_inline__))
simd_veqvl (longv4 __A, longv4 __B)
{
        return (longv4)__builtin_sw_veqvl(__A, __B);
}
#endif
#endif

//vandw
static __inline intv8 __attribute__ ((__always_inline__))
simd_vandw (intv8 __A, intv8 __B)
{
  return (intv8) (__A & __B);
}
//vbicw
static __inline intv8 __attribute__ ((__always_inline__))
simd_vbicw (intv8 __A, intv8 __B)
{
  return (intv8) (__A &~ __B);
}
//vbisw
static __inline intv8 __attribute__ ((__always_inline__))
simd_vbisw (intv8 __A, intv8 __B)
{
  return (intv8) (__A | __B);
}
//vornotw
static __inline intv8 __attribute__ ((__always_inline__))
simd_vornotw (intv8 __A, intv8 __B)
{
  return (intv8) (__A |~ __B);
}
//vxorw
static __inline intv8 __attribute__ ((__always_inline__))
simd_vxorw (intv8 __A, intv8 __B)
{
  return (intv8) (__A ^ __B);
}
//veqvw
static __inline intv8 __attribute__ ((__always_inline__))
simd_veqvw (intv8 __A, intv8 __B)
{
  return (intv8) __builtin_sw_veqvw (__A, __B);
}


/******************************************************************************/
/*                        FP Arithmetic Instruction                           */
/******************************************************************************/
//VADDS
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vadds (floatv4 __A, floatv4 __B)
{
  return (floatv4)(__A + __B);
}
//VADDD
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vaddd (doublev4 __A, doublev4 __B)
{
  return (doublev4)(__A + __B);
}
//VSUBS
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vsubs (floatv4 __A, floatv4 __B)
{
  return (floatv4)(__A - __B);
}
//VSUBD
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vsubd (doublev4 __A, doublev4 __B)
{
  return (doublev4)(__A - __B);
}
//VMULS
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vmuls (floatv4 __A, floatv4 __B)
{
  return (floatv4)(__A * __B);
}
//VMULD
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vmuld (doublev4 __A, doublev4 __B)
{
  return (doublev4)(__A * __B);
}
//VDIVS
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vdivs (floatv4 __A, floatv4 __B)
{
  return (floatv4)(__A / __B);
}
//VDIVD
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vdivd (doublev4 __A, doublev4 __B)
{
  return (doublev4)(__A / __B);
}
//VSQRTS
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vsqrts (floatv4 __x)
{
  return (floatv4) __builtin_sw_vsqrts (__x);
}
//VSQRTD
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vsqrtd (doublev4 __x)
{
  return (doublev4) __builtin_sw_vsqrtd (__x);
}

#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
//vsums vsumd
static __inline float __attribute__((__always_inline__))
simd_vsums (floatv4 __A)
{
        return (float)__builtin_sw_vsums(__A);
}

static __inline double __attribute__((__always_inline__))
simd_vsumd (doublev4 __A)
{
        return (double)__builtin_sw_vsumd(__A);
}

//vfrecs vfrecd
static __inline floatv4 __attribute__((__always_inline__))
simd_vfrecs (floatv4 __A)
{
        return (floatv4)__builtin_sw_vfrecs(__A);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vfrecd (doublev4 __A)
{
        return (doublev4)__builtin_sw_vfrecd(__A);
}
#endif
#endif

//simd_vfcmpeqs(floatv4,floatv4)
static __inline floatv4 __attribute__((__always_inline__))
simd_vfcmpeqs (floatv4 __A, floatv4 __B)
{
        return (floatv4)(__A == __B);
}
//simd_vfcmpeqd(doublev4,doublev4)
static __inline doublev4 __attribute__((__always_inline__))
simd_vfcmpeqd (doublev4 __A, doublev4 __B)
{
        return (doublev4)(__A == __B);
}
//simd_vfcmples(floatv4,floatv4)
static __inline floatv4 __attribute__((__always_inline__))
simd_vfcmples (floatv4 __A, floatv4 __B)
{
        return (floatv4)(__A <= __B);
}
//simd_vfcmpled(doublev4,doublev4)
static __inline doublev4 __attribute__((__always_inline__))
simd_vfcmpled (doublev4 __A, doublev4 __B)
{
        return (doublev4)(__A <= __B);
}
//simd_vfcmplts(floatv4,floatv4)
static __inline floatv4 __attribute__((__always_inline__))
simd_vfcmplts (floatv4 __A, floatv4 __B)
{
        return (floatv4)(__A < __B);
}
//simd_vfcmpltd(doublev4,doublev4)
static __inline doublev4 __attribute__((__always_inline__))
simd_vfcmpltd (doublev4 __A, doublev4 __B)
{
        return (doublev4)(__A < __B);
}
//simd_vfcmpuns(floatv4,floatv4)
static __inline floatv4 __attribute__((__always_inline__))
simd_vfcmpuns (floatv4 __A, floatv4 __B)
{
        return (floatv4)__builtin_sw_vfcmpuns(__A, __B);
}
//simd_vfcmpund(doublev4,doublev4)
static __inline doublev4 __attribute__((__always_inline__))
simd_vfcmpund (doublev4 __A, doublev4 __B)
{
        return (doublev4)__builtin_sw_vfcmpund(__A, __B);
}

#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
//vfcvtsd vfcvtds 
static __inline doublev4 __attribute__((__always_inline__))
simd_vfcvtsd (floatv4 __A)
{
        return (doublev4)__builtin_sw_vfcvtsd(__A);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vfcvtds (doublev4 __A)
{
        return (floatv4)__builtin_sw_vfcvtds(__A);
}

//vfcvtsh vfcvths 
static __inline doublev4 __attribute__((__always_inline__))
simd_vfcvtsh (floatv4 __A, floatv4 __B, const int __C)
{
        return (doublev4)__builtin_sw_vfcvtsh(__A, __B, __C);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vfcvths (doublev4 __A, const int __B)
{
        return (floatv4)__builtin_sw_vfcvths(__A, __B);
}

//vfcvtls vfcvtsl
static __inline floatv4 __attribute__((__always_inline__))
simd_vfcvtls (longv4 __A)
{
        return (floatv4)__builtin_sw_vfcvtls(__A);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vfcvtsl (floatv4 __A)
{
        return (longv4)__builtin_sw_vfcvtsl(__A);
}

//vfcvtld vfcvtdl
static __inline doublev4 __attribute__((__always_inline__))
simd_vfcvtld (longv4 __A)
{
        return (doublev4)__builtin_sw_vfcvtld(__A);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vfcvtdl (doublev4 __A)
{
        return (longv4)__builtin_sw_vfcvtdl(__A);
}

//vfcvtdl_g _p _z _n
static __inline longv4 __attribute__((__always_inline__))
simd_vfcvtdl_g (doublev4 __A)
{
        return (longv4)__builtin_sw_vfcvtdl_g(__A);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vfcvtdl_p (doublev4 __A)
{
        return (longv4)__builtin_sw_vfcvtdl_p(__A);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vfcvtdl_z (doublev4 __A)
{
        return (longv4)__builtin_sw_vfcvtdl_z(__A);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vfcvtdl_n (doublev4 __A)
{
        return (longv4)__builtin_sw_vfcvtdl_n(__A);
}

//vfris _g _p _z _n
static __inline floatv4 __attribute__((__always_inline__))
simd_vfris (floatv4 __A)
{
        return (floatv4)__builtin_sw_vfris(__A);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vfris_g (floatv4 __A)
{
        return (floatv4)__builtin_sw_vfris_g(__A);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vfris_p (floatv4 __A)
{
        return (floatv4)__builtin_sw_vfris_p(__A);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vfris_z (floatv4 __A)
{
        return (floatv4)__builtin_sw_vfris_z(__A);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vfris_n (floatv4 __A)
{
        return (floatv4)__builtin_sw_vfris_n(__A);
}

//vfrid _g _p _z _n
static __inline doublev4 __attribute__((__always_inline__))
simd_vfrid (doublev4 __A)
{
        return (doublev4)__builtin_sw_vfrid(__A);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vfrid_g (doublev4 __A)
{
        return (doublev4)__builtin_sw_vfrid_g(__A);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vfrid_p (doublev4 __A)
{
        return (doublev4)__builtin_sw_vfrid_p(__A);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vfrid_z (doublev4 __A)
{
        return (doublev4)__builtin_sw_vfrid_z(__A);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vfrid_n (doublev4 __A)
{
        return (doublev4)__builtin_sw_vfrid_n(__A);
}
#endif
static __inline floatv4 __attribute__((__always_inline__))
simd_vmaxs (floatv4 __A, floatv4 __B)
{
        return (floatv4)__builtin_sw_smaxs(__A, __B);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vmins (floatv4 __A, floatv4 __B)
{
        return (floatv4)__builtin_sw_smins(__A, __B);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vmaxd (doublev4 __A, doublev4 __B)
{
        return (doublev4)__builtin_sw_smaxd(__A, __B);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vmind (doublev4 __A, doublev4 __B)
{
        return (doublev4)__builtin_sw_smind(__A, __B);
}
#endif

//simd_vcpyss(floatv4,floatv4)
static __inline floatv4 __attribute__((__always_inline__))
simd_vcpyss (floatv4 __A, floatv4 __B) 
{
        return (floatv4)__builtin_sw_vcpyss(__A, __B);
}
//simd_vcpysd(doublev4,doublev4)
static __inline doublev4 __attribute__((__always_inline__))
simd_vcpysd (doublev4 __A, doublev4 __B) 
{
        return (doublev4)__builtin_sw_vcpysd(__A, __B);
}
//simd_vcpyses(floatv4,floatv4)
static __inline floatv4 __attribute__((__always_inline__))
simd_vcpyses (floatv4 __A, floatv4 __B) 
{
        return (floatv4)__builtin_sw_vcpyses(__A, __B);
}
//simd_vcpysed(doublev4,doublev4)
static __inline doublev4 __attribute__((__always_inline__))
simd_vcpysed (doublev4 __A, doublev4 __B) 
{
        return (doublev4)__builtin_sw_vcpysed(__A, __B);
}
//simd_vcpysns(floatv4,floatv4)
static __inline floatv4 __attribute__((__always_inline__))
simd_vcpysns (floatv4 __A, floatv4 __B) 
{
        return (floatv4)__builtin_sw_vcpysns(__A, __B);
}
//simd_vcpysnd(fdoublev4,doublev4)
static __inline doublev4 __attribute__((__always_inline__))
simd_vcpysnd (doublev4 __A, doublev4 __B) 
{
        return (doublev4)__builtin_sw_vcpysnd(__A, __B);
}
//simd_vmas(floatv4,floatv4,floatv4)
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vmas (floatv4 __A, floatv4 __B, floatv4 __C)
{
  return (__A * __B + __C);
}
//simd_vmad(dloublev4,doublev4,doublev4)
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vmad (doublev4 __A, doublev4 __B, doublev4 __C)
{
  return (__A * __B + __C);
}
//simd_vmss(floatv4,floatv4,floatv4)
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vmss (floatv4 __A, floatv4 __B, floatv4 __C)
{
  return (__A * __B - __C);
}
//simd_vmsd(dloublev4,doublev4,doublev4)
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vmsd (doublev4 __A, doublev4 __B, doublev4 __C)
{
  return (__A * __B - __C);
}
//simd_vnmas(floatv4,floatv4,floatv4)
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vnmas (floatv4 __A, floatv4 __B, floatv4 __C)
{
  return (- __A * __B + __C);
}
//simd_vnmad(dloublev4,doublev4,doublev4)
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vnmad (doublev4 __A, doublev4 __B, doublev4 __C)
{
  return (- __A * __B + __C);
}
//simd_vnmss(floatv4,floatv4,floatv4)
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vnmss (floatv4 __A, floatv4 __B, floatv4 __C)
{
  return (- __A * __B - __C);
}
//simd_vnmsd(dloublev4,doublev4,doublev4)
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vnmsd (doublev4 __A, doublev4 __B, doublev4 __C)
{
  return (- __A * __B - __C);
}
//simd_vfseleqs(floatv4,floatv4,floatv4)
static __inline floatv4 __attribute__((__always_inline__))
simd_vfseleqs (floatv4 __A, floatv4 __B, floatv4 __C) 
{
  return (floatv4)__builtin_sw_vfseleqs (__A, __B, __C);
}
//simd_vfseleqd(dloublev4,doublev4,doublev4)
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vfseleqd (doublev4 __A, doublev4 __B, doublev4 __C)
{
  return (doublev4)__builtin_sw_vfseleqd (__A, __B, __C);
}
//simd_vfsellts(floatv4,floatv4,floatv4)
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vfsellts (floatv4 __A, floatv4 __B, floatv4 __C)
{
  return (floatv4)__builtin_sw_vfsellts (__A, __B, __C);
}
//simd_vfselltd(dloublev4,doublev4,doublev4)
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vfselltd (doublev4 __A, doublev4 __B, doublev4 __C)
{
  return (doublev4)__builtin_sw_vfselltd (__A, __B, __C);
}
//simd_vfselles(floatv4,floatv4,floatv4)
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vfselles (floatv4 __A, floatv4 __B, floatv4 __C)
{
  return (floatv4)__builtin_sw_vfselles (__A, __B, __C);
}
//simd_vfselled(dloublev4,doublev4,doublev4)
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vfselled (doublev4 __A, doublev4 __B, doublev4 __C)
{
  return (doublev4)__builtin_sw_vfselled (__A, __B, __C);
}

/******************************************************************************/
/*                        Date Reduction Instruction                          */
/******************************************************************************/
//simd_vinsw(int,intv8,constint)
//simd_vinswn(int,intv8)
//n=(0,1,…,7)
static __inline intv8 __attribute__ ((__always_inline__))
simd_vinsw (int __A, intv8 __B, const int __C)
{
  return (intv8)__builtin_sw_vinsw (__A, __B, __C);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vinsw0 (int __A, intv8 __B)
{
  return (intv8)__builtin_sw_vinsw (__A, __B, 0);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vinsw1 (int __A, intv8 __B)
{
  return (intv8)__builtin_sw_vinsw (__A, __B, 1);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vinsw2 (int __A, intv8 __B)
{
  return (intv8)__builtin_sw_vinsw (__A, __B, 2);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vinsw3 (int __A, intv8 __B)
{
  return (intv8)__builtin_sw_vinsw (__A, __B, 3);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vinsw4 (int __A, intv8 __B)
{
  return (intv8)__builtin_sw_vinsw (__A, __B, 4);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vinsw5 (int __A, intv8 __B)
{
  return (intv8)__builtin_sw_vinsw (__A, __B, 5);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vinsw6 (int __A, intv8 __B)
{
  return (intv8)__builtin_sw_vinsw (__A, __B, 6);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_vinsw7 (int __A, intv8 __B)
{
  return (intv8)__builtin_sw_vinsw (__A, __B, 7);
}
//simd_vinsfs(float,floatv4,constint)
//simd_vinsfsn(float,floatv4)
//n=(0,1,…,7)
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vinsfs (float __A, floatv4 __B, const int __C)
{
  return (floatv4)__builtin_sw_vinsfs (__A, __B, __C);
}
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vinsfs0 (float __A, floatv4 __B)
{
  return (floatv4)__builtin_sw_vinsfs (__A, __B, 0);
}
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vinsfs1 (float __A, floatv4 __B)
{
  return (floatv4)__builtin_sw_vinsfs (__A, __B, 1);
}
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vinsfs2 (float __A, floatv4 __B)
{
  return (floatv4)__builtin_sw_vinsfs (__A, __B, 2);
}
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vinsfs3 (float __A, floatv4 __B)
{
  return (floatv4)__builtin_sw_vinsfs (__A, __B, 3);
}
//simd_vinsfd(double,doublev4,constint)
//simd_vinsfdn(double,doublev4)
//n=(0,1,…,7)
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vinsfd (double __A, doublev4 __B, const int __C)
{
  return (doublev4)__builtin_sw_vinsfd (__A, __B, __C);
}
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vinsfd0 (double __A, doublev4 __B)
{
  return (doublev4)__builtin_sw_vinsfd (__A, __B, 0);
}
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vinsfd1 (double __A, doublev4 __B)
{
  return (doublev4)__builtin_sw_vinsfd (__A, __B, 1);
}
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vinsfd2 (double __A, doublev4 __B)
{
  return (doublev4)__builtin_sw_vinsfd (__A, __B, 2);
}
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vinsfd3 (double __A, doublev4 __B)
{
  return (doublev4)__builtin_sw_vinsfd (__A, __B, 3);
}
//simd_vextw(intv8,constint)
//simd_vextwn(intv8)
//n=(0,1,…,7)
static __inline int __attribute__ ((__always_inline__))
simd_vextw (intv8 __A, const int __B)
{
  return (int)__builtin_sw_vextw (__A, __B);
}

static __inline int __attribute__ ((__always_inline__))
simd_vextw0 (intv8 __A)
{
  return (int)__builtin_sw_vextw (__A, 0);
}

static __inline int __attribute__ ((__always_inline__))
simd_vextw1 (intv8 __A)
{
  return (int)__builtin_sw_vextw (__A, 1);
}
static __inline int __attribute__ ((__always_inline__))
simd_vextw2 (intv8 __A)
{
  return (int)__builtin_sw_vextw (__A, 2);
}
static __inline int __attribute__ ((__always_inline__))
simd_vextw3 (intv8 __A)
{
  return (int)__builtin_sw_vextw (__A, 3);
}
static __inline int __attribute__ ((__always_inline__))
simd_vextw4 (intv8 __A)
{
  return (int)__builtin_sw_vextw (__A, 4);
}

static __inline int __attribute__ ((__always_inline__))
simd_vextw5 (intv8 __A)
{
  return (int)__builtin_sw_vextw (__A, 5);
}

static __inline int __attribute__ ((__always_inline__))
simd_vextw6 (intv8 __A)
{
  return (int)__builtin_sw_vextw (__A, 6);
}

static __inline int __attribute__ ((__always_inline__))
simd_vextw7 (intv8 __A)
{
  return (int)__builtin_sw_vextw (__A, 7);
}
//simd_vextfs(floatv4,constint)
//simd_vextfsn(floatv4)
//n=(0,1,…,7)
static __inline float __attribute__ ((__always_inline__))
simd_vextfs (floatv4 __A, const int __B)
{
  return (float)__builtin_sw_vextfs (__A, __B);
}
static __inline float __attribute__ ((__always_inline__))
simd_vextfs0 (floatv4 __A)
{
  return (float)__builtin_sw_vextfs (__A, 0);
}
static __inline float __attribute__ ((__always_inline__))
simd_vextfs1 (floatv4 __A)
{
  return (float)__builtin_sw_vextfs (__A, 1);
}
static __inline float __attribute__ ((__always_inline__))
simd_vextfs2 (floatv4 __A)
{
  return (float)__builtin_sw_vextfs (__A, 2);
}
static __inline float __attribute__ ((__always_inline__))
simd_vextfs3 (floatv4 __A)
{
  return (float)__builtin_sw_vextfs (__A, 3);
}
//simd_vextfd(doublev4,constint)
//simd_vextfdn(doublev4)
//n=(0,1,…,7)
static __inline double __attribute__ ((__always_inline__))
simd_vextfd (doublev4 __A, const int __B)
{
  return (double)__builtin_sw_vextfd (__A, __B);
}
static __inline double __attribute__ ((__always_inline__))
simd_vextfd0 (doublev4 __A)
{
  return (double)__builtin_sw_vextfd (__A, 0);
}
static __inline double __attribute__ ((__always_inline__))
simd_vextfd1 (doublev4 __A)
{
  return (double)__builtin_sw_vextfd (__A, 1);
}
static __inline double __attribute__ ((__always_inline__))
simd_vextfd2 (doublev4 __A)
{
  return (double)__builtin_sw_vextfd (__A, 2);
}
static __inline double __attribute__ ((__always_inline__))
simd_vextfd3 (doublev4 __A)
{
  return (double)__builtin_sw_vextfd (__A, 3);
}
//simd_vcpyw(int)
static __inline intv8 __attribute__ ((__always_inline__))
simd_vcpyw (int __A)
{
  return (intv8)__builtin_sw_vcpyw (__A);
}
//simd_vcpyfs(float)
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vcpyfs (float __A)
{
  return (floatv4)__builtin_sw_vcpyfs (__A);
}
//simd_vcpyfd(double)
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vcpyfd (double __A)
{
  return (doublev4)__builtin_sw_vcpyfd (__A);
}
//simd_vconw(intv8,intv8,void*)
static __inline intv8 __attribute__ ((__always_inline__))
simd_vconw (intv8 __A, intv8 __B,  void * __C)
{
  return (intv8)__builtin_sw_vconw (__A, __B, __C);
}
//simd_vshfw(intv8,intv8,int)
static __inline intv8 __attribute__ ((__always_inline__))
simd_vshfw (intv8 __A, intv8 __B, int __C)
{
  return (intv8)__builtin_sw_vshfw (__A, __B, __C);
}
//simd_vcons(floatv4,floatv4,void*)
static __inline floatv4 __attribute__ ((__always_inline__))
simd_vcons (floatv4 __A, floatv4 __B, void * __C)
{
  return (floatv4)__builtin_sw_vcons (__A, __B, __C);
}
//simd_vcond(doublev4,doublev4,void*)
static __inline doublev4 __attribute__ ((__always_inline__))
simd_vcond (doublev4 __A, doublev4 __B, void * __C)
{
  return (doublev4)__builtin_sw_vcond (__A, __B, __C);
}

#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
//vinsb vinsh
static __inline charv32 __attribute__((__always_inline__))
simd_vinsb (char __A, charv32 __B, const int __C)
{
        return (charv32)__builtin_sw_vinsb(__A, __B, __C);
}

static __inline shortv16 __attribute__((__always_inline__))
simd_vinsh (short __A, shortv16 __B, const int __C)
{
        return (shortv16)__builtin_sw_vinsh(__A, __B, __C);
}

//vinsectlb vinsectlh vinsectlw vinsectll
static __inline charv32 __attribute__((__always_inline__))
simd_vinsectlb (charv32 __A, charv32 __B)
{
        return (charv32)__builtin_sw_vinsectlb(__A, __B);
}

static __inline shortv16 __attribute__((__always_inline__))
simd_vinsectlh (shortv16 __A, shortv16 __B)
{
        return (shortv16)__builtin_sw_vinsectlh(__A, __B);
}

static __inline intv8 __attribute__((__always_inline__))
simd_vinsectlw (intv8 __A, intv8 __B)
{
        return (intv8)__builtin_sw_vinsectlw(__A, __B);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vinsectll (longv4 __A, longv4 __B)
{
        return (longv4)__builtin_sw_vinsectll(__A, __B);
}

//vshfq vshfqb
static __inline intv8 __attribute__((__always_inline__))
simd_vshfq (intv8 __A, intv8 __B, const int __C)
{
        return (intv8)__builtin_sw_vshfq(__A, __B, __C);
}

static __inline charv32 __attribute__((__always_inline__))
simd_vshfqb (charv32 __A, charv32 __B)
{
        return (charv32)__builtin_sw_vshfqb(__A, __B);
}

//vcpyb vcpyh
static __inline charv32 __attribute__((__always_inline__))
simd_vcpyb (char __A)
{
        return (charv32)__builtin_sw_vcpyb(__A);
}

static __inline shortv16 __attribute__((__always_inline__))
simd_vcpyh (short __A)
{
        return (shortv16)__builtin_sw_vcpyh(__A);
}

/* Collate operation for longv4.  */
static __inline longv4 __attribute__((__always_inline__))
simd_vinsl (long __A, longv4 __B, const int __C)
{
        return (longv4)__builtin_sw_vinsl(__A, __B, __C);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vinsl0 (long __A, longv4 __B)
{
        return (longv4)__builtin_sw_vinsl(__A, __B, 0);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vinsl1 (long __A, longv4 __B)
{
        return (longv4)__builtin_sw_vinsl(__A, __B, 1);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vinsl2 (long __A, longv4 __B)
{
        return (longv4)__builtin_sw_vinsl(__A, __B, 2);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vinsl3 (long __A, longv4 __B)
{
        return (longv4)__builtin_sw_vinsl(__A, __B, 3);
}

static __inline long __attribute__((__always_inline__))
simd_vextl (longv4 __A, const int __B)
{
        return (long)__builtin_sw_vextl(__A, __B);
}

static __inline long __attribute__((__always_inline__))
simd_vextl0 (longv4 __A)
{
        return (long)__builtin_sw_vextl(__A, 0);
}

static __inline long __attribute__((__always_inline__))
simd_vextl1 (longv4 __A)
{
        return (long)__builtin_sw_vextl(__A, 1);
}

static __inline long __attribute__((__always_inline__))
simd_vextl2 (longv4 __A)
{
        return (long)__builtin_sw_vextl(__A, 2);
}

static __inline long __attribute__((__always_inline__))
simd_vextl3 (longv4 __A)
{
        return (long)__builtin_sw_vextl(__A, 3);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vcpyl (long __A)
{
        return (longv4)__builtin_sw_vcpyl(__A);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vconl (longv4 __A, longv4 __B, void * __C)
{
        return (longv4)__builtin_sw_vconl(__A, __B, __C);
}
#endif
#endif

/******************************************************************************/
/*                        PIN Security Instruction                            */
/******************************************************************************/
//vsm3msw
#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
static __inline intv8 __attribute__((__always_inline__))
simd_vsm3msw (intv8 __A, intv8 __B)
{
        return (intv8)__builtin_sw_vsm3msw(__A, __B);
}
//vsm3r
static __inline intv8 __attribute__((__always_inline__))
simd_vsm3r (intv8 __A, intv8 __B, const int __C)
{
        return (intv8)__builtin_sw_vsm3r(__A, __B, __C);
}
//vsm4key
static __inline intv8 __attribute__((__always_inline__))
simd_vsm4key (intv8 __A, const int __B)
{
        return (intv8)__builtin_sw_vsm4key(__A, __B);
}
//vsm4r
static __inline intv8 __attribute__((__always_inline__))
simd_vsm4r (intv8 __A, intv8 __B)
{
        return (intv8)__builtin_sw_vsm4r(__A, __B);
}
#endif
#endif

#ifdef SHIPS20220228_SW8A_SM2&&AES&&SHA
#ifdef __sw_64_sw8a__
static __inline int256 __attribute__ ((__always_inline__))
simd_addow (int256 __A, int256 __B)
{
  return (int256)__builtin_sw_addow (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_addowi (int256 __A, const int __B)
{
  return (int256)__builtin_sw_addowi (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_subow (int256 __A, int256 __B)
{
  return (int256)__builtin_sw_subow (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_subowi (int256 __A, const int __B)
{
  return (int256)__builtin_sw_subowi (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_cmpeqow (int256 __A, int256 __B)
{
  return (int256)__builtin_sw_cmpeqow (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_cmpuleow (int256 __A, int256 __B)
{
  return (int256)__builtin_sw_cmpuleow (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_cmpultow (int256 __A, int256 __B)
{
  return (int256)__builtin_sw_cmpultow (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_umulow (int256 __A, int256 __B)
{
  return (int256)__builtin_sw_umulow (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_vaesenc (int256 __A, int256 __B)
{
  return (int256)__builtin_sw_vaesenc (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_vaesencl (int256 __A, int256 __B)
{
  return (int256)__builtin_sw_vaesencl (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_vaesdec (int256 __A, int256 __B)
{
  return (int256)__builtin_sw_vaesdec (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_vaesdecl (int256 __A, int256 __B)
{
  return (int256)__builtin_sw_vaesdecl (__A, __B);
}
static __inline int256 __attribute__ ((__always_inline__))
simd_vaessbox (int256 __A)
{
  return (int256)__builtin_sw_vaessbox (__A);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_sha1msw (intv8 __A, intv8 __B)
{
  return (intv8)__builtin_sw_sha1msw (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_sha1r (intv8 __A, intv8 __B, const int __C)
{
  return (intv8)__builtin_sw_sha1r (__A, __B, __C);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_sha256msw (intv8 __A, intv8 __B)
{
  return (intv8)__builtin_sw_sha256msw (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_sha256r (intv8 __A, intv8 __B, const int __C)
{
  return (intv8)__builtin_sw_sha256r (__A, __B, __C);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_sha512msl0 (intv8 __A, intv8 __B)
{
  return (intv8)__builtin_sw_sha512msl0 (__A, __B);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_sha512msl1 (intv8 __A, intv8 __B, intv8 __C)
{
  return (intv8)__builtin_sw_sha512msl1 (__A, __B, __C);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_sha512r0 (intv8 __A, intv8 __B, intv8 __C)
{
  return (intv8)__builtin_sw_sha512r0 (__A, __B, __C);
}
static __inline intv8 __attribute__ ((__always_inline__))
simd_sha512r1 (intv8 __A, intv8 __B, intv8 __C)
{
  return (intv8)__builtin_sw_sha512r1 (__A, __B, __C);
}
#endif
#endif
/******************************************************************************/
/*                        Extend Instruction                                  */
/******************************************************************************/
//simd_fprint_intv8(FILE*,intv8)
static __inline void
simd_fprint_intv8 (FILE *fp, intv8 a)
{
  union {
    int __a[8];
    intv8 __v;
  } __u;
  __u.__v = a;
  fprintf (fp, "[ %d, %d, %d, %d, %d, %d, %d, %d ]\n", 
		  __u.__a[7], __u.__a[6], __u.__a[5], 
		  __u.__a[4], __u.__a[3], __u.__a[2], 
		  __u.__a[1], __u.__a[0]);
}
//simd_fprint_uintv8(FILE*,uintv8)
static __inline void
simd_fprint_uintv8 (FILE *fp, uintv8 a)
{
  union {
    unsigned int __a[8];
    uintv8 __v;
  } __u;
  __u.__v = a;
  fprintf (fp, "[ %u, %u, %u, %u, %u, %u, %u, %u ]\n", 
		  __u.__a[7], __u.__a[6], __u.__a[5], 
		  __u.__a[4], __u.__a[3], __u.__a[2], 
		  __u.__a[1], __u.__a[0]);
}
//simd_fprint_floatv4(FILE*,floatv4)
static __inline void
simd_fprint_floatv4 (FILE *fp, floatv4 a)
{
  union {
    float __a[4];
    floatv4 __v;
  } __u;
  __u.__v = a;
  fprintf (fp, "[ %.8e, %.8e, %.8e, %.8e ]\n", 
		  __u.__a[3], __u.__a[2], __u.__a[1], __u.__a[0]);
}
//simd_fprint_doublev4(FILE*,doublev4)
static __inline void
simd_fprint_doublev4 (FILE *fp, doublev4 a)
{
  union {
    double __a[4];
    doublev4 __v;
  } __u;
  __u.__v = a;
  fprintf (fp, "[ %.16e, %.16e, %.16e, %.16e ]\n", 
		  __u.__a[3], __u.__a[2], __u.__a[1], __u.__a[0]);
}
//simd_fprint_int256(FILE*,int256)
static __inline void
simd_fprint_int256 (FILE *fp, int256 a)
{
  volatile union {
    long __a[4];
    int256 __v;
  } __u;
  __u.__v = a;
  fprintf (fp, "[ 0x%lx, 0x%lx, 0x%lx, 0x%lx ]\n", 
		  __u.__a[3], __u.__a[2], __u.__a[1], __u.__a[0]);
}

//simd_fprint_uint256(FILE*,uint256)
static __inline void
simd_fprint_uint256 (FILE *fp, uint256 a)
{
  volatile union {
    unsigned long __a[4];
    uint256 __v;
  } __u;
  __u.__v = a;
  fprintf (fp, "[ 0x%lx, 0x%lx, 0x%lx, 0x%lx ]\n", 
		  __u.__a[3], __u.__a[2], __u.__a[1], __u.__a[0]);
}

#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
static __inline void
simd_fprint_longv4 (FILE *fp, longv4 a)
{
  union {
    long __a[4];
    longv4 __v;
  } __u;
  __u.__v = a;  fprintf (fp, "[ %ld, %ld, %ld, %ld ]\n", __u.__a[3], __u.__a[2], __u.__a[1], __u.__a[0]);
}

static __inline void
simd_fprint_ulongv4 (FILE *fp, ulongv4 a)
{
  union {
    unsigned long __a[4];
    ulongv4 __v;
  } __u;
  __u.__v = a;
  fprintf (fp, "[ %u, %u, %u, %u ]\n", __u.__a[3], __u.__a[2], __u.__a[1], __u.__a[0]);
}

static __inline void
simd_fprint_charv32 (FILE *fp, charv32 a)
{
  union {
    char __a[32];
    charv32 __v;
  } __u;
  __u.__v = a;  fprintf (fp, "[ %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d ]\n", __u.__a[31], __u.__a[30], __u.__a[29], __u.__a[28], __u.__a[27], __u.__a[26], __u.__a[25], __u.__a[24], __u.__a[23], __u.__a[22], __u.__a[21], __u.__a[20], __u.__a[19], __u.__a[18], __u.__a[17], __u.__a[16],  __u.__a[15], __u.__a[14], __u.__a[13], __u.__a[12], __u.__a[11], __u.__a[10], __u.__a[9], __u.__a[8], __u.__a[7], __u.__a[6], __u.__a[5], __u.__a[4], __u.__a[3], __u.__a[2],__u.__a[1], __u.__a[0]);
}


static __inline void
simd_fprint_ucharv32 (FILE *fp, ucharv32 a)
{
  union {
    unsigned char __a[32];
    ucharv32 __v;
  } __u;
  __u.__v = a;
  fprintf (fp, "[ %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u ]\n", __u.__a[31], __u.__a[30], __u.__a[29], __u.__a[28], __u.__a[27], __u.__a[26], __u.__a[25], __u.__a[24], __u.__a[23], __u.__a[22], __u.__a[21], __u.__a[20], __u.__a[19], __u.__a[18], __u.__a[17], __u.__a[16],  __u.__a[15], __u.__a[14], __u.__a[13], __u.__a[12], __u.__a[11], __u.__a[10], __u.__a[9], __u.__a[8], __u.__a[7], __u.__a[6], __u.__a[5], __u.__a[4], __u.__a[3], __u.__a[2],__u.__a[1], __u.__a[0]);
}

static __inline void
simd_fprint_shortv16 (FILE *fp, shortv16 a)
{
  union {
    short __a[16];
    shortv16 __v;
  } __u;
  __u.__v = a;  fprintf (fp, "[ %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d ]\n",  __u.__a[15], __u.__a[14], __u.__a[13], __u.__a[12], __u.__a[11], __u.__a[10], __u.__a[9],__u.__a[8], __u.__a[7], __u.__a[6], __u.__a[5], __u.__a[4], __u.__a[3], __u.__a[2], __u.__a[1], __u.__a[0]);
}

static __inline void
simd_fprint_ushortv16 (FILE *fp, ushortv16 a)
{
  union {
    unsigned short __a[16];
    ushortv16 __v;
  } __u;
  __u.__v = a;
  fprintf (fp, "[ %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u ]\n",  __u.__a[15], __u.__a[14], __u.__a[13], __u.__a[12], __u.__a[11], __u.__a[10], __u.__a[9],__u.__a[8], __u.__a[7], __u.__a[6], __u.__a[5], __u.__a[4], __u.__a[3], __u.__a[2], __u.__a[1], __u.__a[0]);
}
#endif
#endif

//simd_set_intv8(int,…,int)
static __inline intv8
simd_set_intv8 (int __S, int __T, int __U, int __V, 
		int __W, int __X, int __Y, int __Z)
{
    union {
    int __a[8] __attribute__ ((aligned (32)));
    intv8 __v;
  } __u;

  __u.__a[0] = __S;
  __u.__a[1] = __T;
  __u.__a[2] = __U;
  __u.__a[3] = __V;
  __u.__a[4] = __W;
  __u.__a[5] = __X;
  __u.__a[6] = __Y;
  __u.__a[7] = __Z;

  return __u.__v;
}

//simd_set_uintv8(unsignedint,…,unsignedint)
static __inline uintv8
simd_set_uintv8 (unsigned int __S, unsigned int __T, unsigned int __U,
	       	 unsigned int __V, unsigned int __W, unsigned int __X, 
		 unsigned int __Y, unsigned int __Z)
{
    union {
    unsigned int __a[8] __attribute__ ((aligned (32)));
    uintv8 __v;
  } __u;

  __u.__a[0] = __S;
  __u.__a[1] = __T;
  __u.__a[2] = __U;
  __u.__a[3] = __V;
  __u.__a[4] = __W;
  __u.__a[5] = __X;
  __u.__a[6] = __Y;
  __u.__a[7] = __Z;

  return __u.__v;
}

//simd_set_floatv4(float,…,flaot)
static __inline floatv4
simd_set_floatv4 (float __W, float __X, float __Y, float __Z)
{
    union {
    float __a[4] __attribute__ ((aligned (32)));
    floatv4 __v;
  } __u;

  __u.__a[0] = __W;
  __u.__a[1] = __X;
  __u.__a[2] = __Y;
  __u.__a[3] = __Z;

  return __u.__v;
}

//simd_set_doublev4(double,…,double)
static __inline doublev4
simd_set_doublev4 (double __W, double __X, double __Y, double __Z)
{
    union {
    double __a[4] __attribute__ ((aligned (32)));
    doublev4 __v;
  } __u;

  __u.__a[0] = __W;
  __u.__a[1] = __X;
  __u.__a[2] = __Y;
  __u.__a[3] = __Z;

  return __u.__v;
}

//simd_set_int256(long,…,long)
static __inline int256
simd_set_int256 (long __W, long __X, long __Y, long __Z)
{
#ifndef XUKK20231024_SWSIMD
    union {
    long __a[4];
    longv4 __v;
  } __u;

  __u.__a[0] = __W;
  __u.__a[1] = __X;
  __u.__a[2] = __Y;
  __u.__a[3] = __Z;

  return __u.__v;
#else 
  int256 __v;
  __v=__builtin_sw_vinsql(__W,__v,0);
  __v=__builtin_sw_vinsql(__X,__v,1);
  __v=__builtin_sw_vinsql(__Y,__v,2);
  __v=__builtin_sw_vinsql(__Z,__v,3);
  return __v;
#endif
}

//simd_set_uint256(unsignedlong,…,unsignedlong)
static __inline uint256
simd_set_uint256 (unsigned long __W, unsigned long __X,
                  unsigned long __Y, unsigned long __Z)
{
#ifndef XUKK20231024_SWSIMD
    union {
    unsigned long __a[4];
    ulongv4 __v;
  } __u;

  __u.__a[0] = __W;
  __u.__a[1] = __X;
  __u.__a[2] = __Y;
  __u.__a[3] = __Z;

  return __u.__v;
#else 
  uint256 __v;
  __v=__builtin_sw_vinsql(__W,__v,0);
  __v=__builtin_sw_vinsql(__X,__v,1);
  __v=__builtin_sw_vinsql(__Y,__v,2);
  __v=__builtin_sw_vinsql(__Z,__v,3);
  return __v;
#endif
}

#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
static __inline longv4
simd_set_longv4 (long __W, long __X, long __Y, long __Z)
{
    union {
    long __a[4];
    longv4 __v;
  } __u;

  __u.__a[0] = __W;
  __u.__a[1] = __X;
  __u.__a[2] = __Y;
  __u.__a[3] = __Z;

  return __u.__v;
}

static __inline ulongv4
simd_set_ulongv4 (unsigned long __W, unsigned long __X, unsigned long __Y, unsigned long __Z)
{
    union {
    unsigned long __a[4];
    ulongv4 __v;
  } __u;

  __u.__a[0] = __W;
  __u.__a[1] = __X;
  __u.__a[2] = __Y;
  __u.__a[3] = __Z;

  return __u.__v;
}

static __inline charv32
simd_set_charv32 (char __Z0, char __Z1, char __Z2, char __Z3, char __Z4, char __Z5, char __Z6, char __Z7, char __Z8, char __Z9, char __Z10, char __Z11, char __Z12, char __Z13, char __Z14, char __Z15, char __Z16, char __Z17, char __Z18, char __Z19, char __Z20, char __Z21, char __Z22, char __Z23, char __Z24, char __Z25, char __Z26, char __Z27, char __Z28,char __Z29, char __Z30, char __Z31)
{
    union {
    char __a[32];
    charv32 __v;
  } __u;

  __u.__a[0] = __Z0;
  __u.__a[1] = __Z1;
  __u.__a[2] = __Z2;
  __u.__a[3] = __Z3;
  __u.__a[4] = __Z4;
  __u.__a[5] = __Z5;
  __u.__a[6] = __Z6;
  __u.__a[7] = __Z7;
  __u.__a[8] = __Z8;
  __u.__a[9] = __Z9;
  __u.__a[10] = __Z10;
  __u.__a[11] = __Z11;
  __u.__a[12] = __Z12;
  __u.__a[13] = __Z13;
  __u.__a[14] = __Z14;
  __u.__a[15] = __Z15;
  __u.__a[16] = __Z16;
  __u.__a[17] = __Z17;
  __u.__a[18] = __Z18;
  __u.__a[19] = __Z19;
  __u.__a[20] = __Z20;
  __u.__a[21] = __Z21;
  __u.__a[22] = __Z22;
  __u.__a[23] = __Z23;
  __u.__a[24] = __Z24;
  __u.__a[25] = __Z25;
  __u.__a[26] = __Z26;
  __u.__a[27] = __Z27;
  __u.__a[28] = __Z28;
  __u.__a[29] = __Z29;
  __u.__a[30] = __Z30;
  __u.__a[31] = __Z31;

  return __u.__v;
}

static __inline ucharv32
simd_set_ucharv32 (unsigned char __Z0, unsigned char __Z1, unsigned char __Z2, unsigned char __Z3, unsigned char __Z4, unsigned char __Z5, unsigned char __Z6, unsigned char __Z7, unsigned char __Z8, unsigned char __Z9, unsigned char __Z10, unsigned char __Z11, unsigned char __Z12, unsigned char __Z13, unsigned char __Z14, unsigned char __Z15, unsigned char __Z16, unsigned char __Z17, unsigned char __Z18, unsigned char __Z19, unsigned char __Z20, unsigned char __Z21, unsigned char __Z22, unsigned char __Z23, unsigned char __Z24,unsigned char __Z25, unsigned char __Z26, unsigned char __Z27, unsigned char __Z28, unsigned char __Z29, unsigned char __Z30, unsigned char __Z31)
{
    union {
    unsigned char __a[32];
    ucharv32 __v;
  } __u;

  __u.__a[0] = __Z0;
  __u.__a[1] = __Z1;
  __u.__a[2] = __Z2;
  __u.__a[3] = __Z3;
  __u.__a[4] = __Z4;
  __u.__a[5] = __Z5;
  __u.__a[6] = __Z6;
  __u.__a[7] = __Z7;
  __u.__a[8] = __Z8;
  __u.__a[9] = __Z9;
  __u.__a[10] = __Z10;
  __u.__a[11] = __Z11;
  __u.__a[12] = __Z12;
  __u.__a[13] = __Z13;
  __u.__a[14] = __Z14;
  __u.__a[15] = __Z15;
  __u.__a[16] = __Z16;
  __u.__a[17] = __Z17;
  __u.__a[18] = __Z18;
  __u.__a[19] = __Z19;
  __u.__a[20] = __Z20;
  __u.__a[21] = __Z21;
  __u.__a[22] = __Z22;
  __u.__a[23] = __Z23;
  __u.__a[24] = __Z24;
  __u.__a[25] = __Z25;
  __u.__a[26] = __Z26;
  __u.__a[27] = __Z27;
  __u.__a[28] = __Z28;
  __u.__a[29] = __Z29;
  __u.__a[30] = __Z30;
  __u.__a[31] = __Z31;

  return __u.__v;
}

static __inline shortv16
simd_set_shortv16 (short __Z0, short __Z1, short __Z2, short __Z3, short __Z4, short __Z5, short __Z6, short __Z7, short __Z8, short __Z9, short __Z10, short __Z11, short __Z12, short __Z13, short __Z14, short __Z15)
{
    union {
    short __a[16];
    shortv16 __v;
  } __u;

  __u.__a[0] = __Z0;
  __u.__a[1] = __Z1;
  __u.__a[2] = __Z2;
  __u.__a[3] = __Z3;
  __u.__a[4] = __Z4;
  __u.__a[5] = __Z5;
  __u.__a[6] = __Z6;
  __u.__a[7] = __Z7;
  __u.__a[8] = __Z8;
  __u.__a[9] = __Z9;
  __u.__a[10] = __Z10;
  __u.__a[11] = __Z11;
  __u.__a[12] = __Z12;
  __u.__a[13] = __Z13;
  __u.__a[14] = __Z14;
  __u.__a[15] = __Z15;

  return __u.__v;
}

static __inline ushortv16
simd_set_ushortv16 (unsigned short __Z0, unsigned short __Z1, unsigned short __Z2, unsigned short __Z3, unsigned short __Z4, unsigned short __Z5, unsigned short __Z6, unsigned short __Z7, unsigned short __Z8, unsigned short __Z9, unsigned short __Z10, unsigned short __Z11, unsigned short __Z12, unsigned short __Z13, unsigned short __Z14, unsigned short __Z15)
{
    union {
    unsigned short __a[16];
    ushortv16 __v;
  } __u;

  __u.__a[0] = __Z0;
  __u.__a[1] = __Z1;
  __u.__a[2] = __Z2;
  __u.__a[3] = __Z3;
  __u.__a[4] = __Z4;
  __u.__a[5] = __Z5;
  __u.__a[6] = __Z6;
  __u.__a[7] = __Z7;
  __u.__a[8] = __Z8;
  __u.__a[9] = __Z9;
  __u.__a[10] = __Z10;
  __u.__a[11] = __Z11;
  __u.__a[12] = __Z12;
  __u.__a[13] = __Z13;
  __u.__a[14] = __Z14;
  __u.__a[15] = __Z15;

  return __u.__v;
}
#endif
#endif

//simd_print_intv8(intv8)
static __inline void simd_print_intv8 (intv8 a) { simd_fprint_intv8 (stdout, a); }
//simd_print_uintv8(uintv8)
static __inline void simd_print_uintv8 (uintv8 a) { simd_fprint_uintv8 (stdout, a); }
//simd_print_int256(int256)
static __inline void simd_print_int256 (int256 a) { simd_fprint_int256 (stdout, a); }
//simd_print_uint256(uint256)
static __inline void simd_print_uint256 (uint256 a) { simd_fprint_uint256 (stdout, a); }
//simd_print_floatv4(floatv4)
static __inline void simd_print_floatv4 (floatv4 a) { simd_fprint_floatv4 (stdout, a); }
//simd_print_doublev4(doublev4)
static __inline void simd_print_doublev4 (doublev4 a) { simd_fprint_doublev4 (stdout, a); }

#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
static __inline void simd_print_longv4 (longv4 a) { simd_fprint_longv4 (stdout, a); }
static __inline void simd_print_ulongv4 (ulongv4 a) { simd_fprint_ulongv4 (stdout, a); }
static __inline void simd_print_charv32 (charv32 a) { simd_fprint_charv32 (stdout, a); }
static __inline void simd_print_ucharv32 (ucharv32 a) { simd_fprint_ucharv32 (stdout, a); }
static __inline void simd_print_shortv16 (shortv16 a) { simd_fprint_shortv16 (stdout, a); }
static __inline void simd_print_ushortv16 (ushortv16 a) { simd_fprint_ushortv16 (stdout, a); }
#endif
#endif
static __inline int __attribute__((__always_inline__))
simd_reduc_plusw (intv8 __A)
{
        return (int) __builtin_sw_reduc_plusw(__A);
}

static __inline float __attribute__((__always_inline__))
simd_reduc_pluss (floatv4 __A)
{
        return (float) __builtin_sw_reduc_pluss(__A);
}

static __inline double __attribute__((__always_inline__))
simd_reduc_plusd (doublev4 __A)
{
        return (double) __builtin_sw_reduc_plusd(__A);
}

static __inline int __attribute__((__always_inline__))
simd_reduc_smaxw (intv8 __A)
{
        return (int) __builtin_sw_reduc_smaxw(__A);
}

static __inline unsigned int __attribute__((__always_inline__))
simd_reduc_umaxw (uintv8 __A)
{
        return (unsigned int) __builtin_sw_reduc_umaxw(__A);
}

static __inline float __attribute__((__always_inline__))
simd_reduc_smaxs (floatv4 __A)
{
        return (float) __builtin_sw_reduc_smaxs(__A);
}

static __inline double __attribute__((__always_inline__))
simd_reduc_smaxd (doublev4 __A)
{
        return (double) __builtin_sw_reduc_smaxd(__A);
}

static __inline int __attribute__((__always_inline__))
simd_reduc_sminw (intv8 __A)
{
        return (int) __builtin_sw_reduc_sminw(__A);
}

static __inline unsigned int __attribute__((__always_inline__))
simd_reduc_uminw (uintv8 __A)
{
        return (unsigned int) __builtin_sw_reduc_uminw(__A);
}

static __inline float __attribute__((__always_inline__))
simd_reduc_smins (floatv4 __A)
{
        return (float) __builtin_sw_reduc_smins(__A);
}

static __inline double __attribute__((__always_inline__))
simd_reduc_smind (doublev4 __A)
{
        return (double) __builtin_sw_reduc_smind(__A);
}


/*************************Float Shift****************************/
static __inline floatv4 __attribute__((__always_inline__))
simd_vslls1 (floatv4 __A)
{
        return (floatv4)  __builtin_sw_vslls(__A, 1);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vslls2 (floatv4 __A)
{
        return (floatv4)  __builtin_sw_vslls(__A, 2);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vslls3 (floatv4 __A)
{
        return (floatv4)  __builtin_sw_vslls(__A, 3);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vslld1 (doublev4 __A)
{
        return (doublev4)  __builtin_sw_vslld(__A, 1);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vslld2 (doublev4 __A)
{
        return (doublev4)  __builtin_sw_vslld(__A, 2);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vslld3 (doublev4 __A)
{
        return (doublev4)  __builtin_sw_vslld(__A, 3);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vsrls1 (floatv4 __A)
{
        return (floatv4)  __builtin_sw_vsrls(__A, 1);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vsrls2 (floatv4 __A)
{
        return (floatv4)  __builtin_sw_vsrls(__A, 2);
}

static __inline floatv4 __attribute__((__always_inline__))
simd_vsrls3 (floatv4 __A)
{
        return (floatv4)  __builtin_sw_vsrls(__A, 3);
}
static __inline doublev4 __attribute__((__always_inline__))
simd_vsrld1 (doublev4 __A)
{
        return (doublev4)  __builtin_sw_vsrld(__A, 1);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vsrld2 (doublev4 __A)
{
        return (doublev4)  __builtin_sw_vsrld(__A, 2);
}

static __inline doublev4 __attribute__((__always_inline__))
simd_vsrld3 (doublev4 __A)
{
        return (doublev4)  __builtin_sw_vsrld(__A, 3);
}

#ifndef XUKK20231214_SW8ASIMD
#ifdef __sw_64_sw8a__
static __inline intv8 __attribute__((__always_inline__))
simd_vbinvw (intv8 __x)
{
	return (intv8) __builtin_sw_vbinvw (__x);
}

static __inline int256 __attribute__((__always_inline__))
simd_vwinv (int256 __x)
{
	return (int256) __builtin_sw_vwinv (__x);
}

static __inline longv4 __attribute__((__always_inline__))
simd_vseleql (longv4 __A, longv4 __B, longv4 __C) 
{
        return (longv4)__builtin_sw_vseleql(__A, __B, __C);
}
#endif
#endif

#ifndef SHIPS20220228_SW8A_CRC32
#ifdef __sw_64_sw8a__
static __inline unsigned int __attribute__ ((__always_inline__))
sw_64_crc32b (unsigned int __A, unsigned char __B)
{
  return (unsigned int)__builtin_sw_crc32b (__A, __B);
}
static __inline unsigned int __attribute__ ((__always_inline__))
sw_64_crc32h (unsigned int __A, unsigned short __B)
{
  return (unsigned int)__builtin_sw_crc32h (__A, __B);
}
static __inline unsigned int __attribute__ ((__always_inline__))
sw_64_crc32w (unsigned int __A, unsigned int __B)
{
  return (unsigned int)__builtin_sw_crc32w (__A, __B);
}
static __inline unsigned long __attribute__ ((__always_inline__))
sw_64_crc32l (unsigned long __A, unsigned long __B)
{
  return (unsigned int)__builtin_sw_crc32l (__A, __B);
}
static __inline unsigned int __attribute__ ((__always_inline__))
sw_64_crc32cb (unsigned int __A, unsigned char __B)
{
  return (unsigned int)__builtin_sw_crc32cb (__A, __B);
}
static __inline unsigned int __attribute__ ((__always_inline__))
sw_64_crc32ch (unsigned int __A, unsigned short __B)
{
  return (unsigned int)__builtin_sw_crc32ch (__A, __B);
}
static __inline unsigned int __attribute__ ((__always_inline__))
sw_64_crc32cw (unsigned int __A, unsigned int __B)
{
  return (unsigned int)__builtin_sw_crc32cw (__A, __B);
}
static __inline unsigned long __attribute__ ((__always_inline__))
sw_64_crc32cl (unsigned long __A, unsigned long __B)
{
  return (unsigned int)__builtin_sw_crc32cl (__A, __B);
}
#endif
#endif

#endif /*_SIMD_H_INCLUDED*/
